// Copyright 2014-2017, Anitoa Systems, LLC
// All rights reserved

#include "stdafx.h"
#include "resource.h"			// main symbols

#include "InterfaceObj.h"
#include "HidMgr.h"
#include "TestBBDlg.h"

extern BYTE TxData[TxNum];		// the buffer of sent data to HID
extern BYTE RxData[RxNum];		// the buffer of received data from HID

extern BOOL g_DeviceDetected;
extern bool	MyDeviceDetected;	// redundant, maybe keep only one?

extern int gain_mode;			// todo: move to member variable

// extern int adc_result[MAX_IMAGE_SIZE][MAX_IMAGE_SIZE];			// this is actually the frame data, not just adc result

//#define pixelsize24 10		// display size for one pixel
//#define pixelsize12 20		// display size for one pixel

// extern int gain_mode;			// 0 high gain, 1 low gain. HW initial high gain mode. However, SW default is low gain

extern float int_time;				// integration time
extern int g_FrameSize;				// 0: 12x12 frame; 1: 24x24 frame

extern BOOL Continue_Flag;
TCHAR g_CurrentDirectory[MAX_PATH];


void CInterfaceObject::SetDlg(CDialogEx* pd) 
{
	pDlg = pd;
}

CString CInterfaceObject::GetChipName()
{
	return m_TrimReader.Node[0].name;
}

/////////////////////////////////////////////////////////////////////////////
// Below are interfaces to set ULS24 internal parameters - called trim data. 
/////////////////////////////////////////////////////////////////////////////

void CInterfaceObject::ResetTrim()
{
	SetRampgen((BYTE)m_TrimReader.Node[0].rampgen); 
	SetRangeTrim(0x0f);

	SetV20(m_TrimReader.Node[0].auto_v20[0]);
	SetV15(m_TrimReader.Node[0].auto_v15);

	SetGainMode(1);			// Low gain
	SetTXbin(0x8);

	SetIntTime(1);			// 1 ms
}

void CInterfaceObject::SetV15(BYTE v15)
{
	m_TrimReader.SetV15(v15);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();
}

void CInterfaceObject::SetV20(BYTE v20)
{
	m_TrimReader.SetV20(v20);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();
}


void CInterfaceObject::SetGainMode(int gain)
{
	m_TrimReader.SetGainMode(gain);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();

	gain_mode = gain;

	((CTestBBDlg*)pDlg)->UpdateGUIDisplay();

	// When gain mode change, V20 needs to change also
	if(!gain) SetV20(m_TrimReader.Node[0].auto_v20[1]); // auto_v20_hg);
	else SetV20(m_TrimReader.Node[0].auto_v20[0]); // auto_v20_lg);
}

void  CInterfaceObject::SetRangeTrim(BYTE range)
{
	m_TrimReader.SetRangeTrim(range);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();
}

void  CInterfaceObject::SetRampgen(BYTE rampgen)
{
	m_TrimReader.SetRampgen(rampgen);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();
}

void  CInterfaceObject::SetTXbin(BYTE txbin)
{
	m_TrimReader.SetTXbin(txbin);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();
}

///////////////////////////////////////////////////////
// Below are routines to adjust integration time
////////////////////////////////////////////////////////

void  CInterfaceObject::SetIntTime(float it) 
{
	m_TrimReader.SetIntTime(it);

	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));
	ReadHIDInputReport();

	int_time = it;

	((CTestBBDlg*)pDlg)->UpdateGUIDisplay();
}

void CInterfaceObject::ProcessRowData()
{
	g_FrameSize = m_TrimReader.ProcessRowData(frame_data, gain_mode);

/*	char fstrbuf[9];
 	CString sADCData;	

	for (int i=0; i<6; i++)
 	{
		itoa (adc_result[5][i], fstrbuf, 10);		//
 		sADCData += fstrbuf;
 		sADCData += " ";
	}

	SetDlgItemText(IDC_STATIC, sADCData);		// for debug
*/
}

int  CInterfaceObject::CaptureFrame12()
{
	// Issue capture command

	m_TrimReader.Capture12();
	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));

	// Read and process result
	Continue_Flag = true;

	while(Continue_Flag) {		// Process data row by row
		ReadHIDInputReport();
		ProcessRowData();
//		((CTestBBDlg*)pDlg)->DrawPattern();
		memset(RxData,0,sizeof(RxData));
	}

	// Application developer can add code here to further process 
	// the data, that is save in "adc_result[24][24]

	return 0;
}

int  CInterfaceObject::CaptureFrame24()
{
		// Issue capture command
	m_TrimReader.Capture24();
	WriteHIDOutputReport();		// 
	memset(TxData,0,sizeof(TxData));

	// Read and process result
	Continue_Flag = true;

	while(Continue_Flag) {		// Process data row by row
		ReadHIDInputReport();
		ProcessRowData();
//		((CTestBBDlg*)pDlg)->DrawPattern();
		memset(RxData,0,sizeof(RxData));
	}

	// Application developer can add code here to further process 
	// the data, that is save in "adc_result[24][24]

	return 0;
}

int  CInterfaceObject::LoadTrimFile()
{		
	GetCurrentDirectory(MAX_PATH, g_CurrentDirectory);

	CString path;
	path = g_CurrentDirectory;
	path += "\\Trim\\trim.dat";

	LPTSTR lpszData = path.GetBuffer(path.GetLength());
	int e = m_TrimReader.Load((TCHAR*)lpszData);
	path.ReleaseBuffer(0);

	if(e) {
		m_TrimReader.Parse();
	}

	return e;
}

int CInterfaceObject::IsDeviceDetected()
{
	return g_DeviceDetected;
}

void CInterfaceObject::DrawImage(int contrast)
{
	if(!g_FrameSize) {
		for(int i=0; i<12; i++) {
			((CTestBBDlg*)pDlg)->DrawPattern(i, contrast);
		}
	}
	else {
		for(int i=0; i<24; i++) {
			((CTestBBDlg*)pDlg)->DrawPattern(i, contrast);
		}
	}
}

void CInterfaceObject::InsertText(CString s)
{
	((CTestBBDlg*)pDlg)->InsertText(s);
}

void CInterfaceObject::DisplayFrameText()
{
	((CTestBBDlg*)pDlg)->DisplayFrameText();
}

